from . import main, suite

# This simply runs the main test suite, so we can call this test suite as a
# module: e.g. "python -m shift.tests"
# NOTE: We need the suite import from above for this to work.
main()

